# # # Sustainability-870974
# # # Henrik Baktoft, Karl Ø. Gjelland, Marcell Szabo-Meszaros, Ana T. Silva, Milan Riha, Finn Økland, Knut Alfredsen and Torbjørn Forseth
# # # Can energy depletion of wild Atlantic salmon kelts negotiating hydropower facilities lead to reduced survival?

# # # This R script creates an artificial data set based on six equations from Lennox et al (2018)
# # # These data are used to obtain a unified model enabling estimation of oxygen consumption from
# # # kelt lenght (LT), water temperature (T) and kelt swimming speed (BLs). See the text for further explanation.

# # # Equations from Lennox et al (2018):
# EQ1:	63.5 cm @ 0 bl * s-1		MO2 = 0.55 * exp(0.07 * T)
# EQ2:	89.0 cm @ 0 bl * s-1		MO2 = 0.45 * exp(0.07 * T)
# EQ3: 119.0 cm @ 0 bl * s-1		MO2 = 0.38 * exp(0.07 * T)
# EQ4:	all sizes @ 0.5 bl * s-1	MO2 = 1.32 * exp(0.03 * T)
# EQ5:	all sizes @ 0.7 bl * s-1	MO2 = 1.85 * exp(0.03 * T)
# EQ6:	all sizes @ 1.0 bl * s-1	MO2 = 2.90 + (0.02 * T) + (0.0021 * T^2)


# # # PRERQUISITES
# # # JAGS needs to be installed befoehand - see http://mcmc-jags.sourceforge.net/ for details
# # # Needed libraries
library(data.table)
library(R2jags)


# # # Create artificial kelt lenghts, water temperature and swimming speeds
# # # And obtain all combinations of these
lng  <- c(63.5, 89.0, 119.0)
temp <- c(0, 5, 10, 15, 20)
bls  <- c(0, 0.5, 0.7, 1.0)

art_dat <- data.table::data.table(expand.grid(lng=lng, temp=temp, bls=bls) )
art_dat[, mo2 := as.numeric(NA)]

# # # Iterate through all rows in art_dat and apply the relevant equation to obtain MO2
for(i in 1:nrow(art_dat)){
	dat_i <- art_dat[i,]
	if(dat_i$bls == 0){
		if(dat_i$lng == 63.5){ 
			# Apply EQ1
			art_dat[i, 'mo2' := 0.55 * exp(0.07 * dat_i$temp)]
		} else if(dat_i$lng == 89.0){ 
			# Apply EQ2
			art_dat[i, 'mo2' := 0.45 * exp(0.07 * dat_i$temp)]
		} else if(dat_i$lng == 119.0){ 
			# Apply EQ3
			art_dat[i, 'mo2':= 0.38 * exp(0.07 * dat_i$temp)]
		}
	} else if(dat_i$bls == 0.5){
		# Apply EQ4
		art_dat[i, 'mo2' := 1.32 * exp(0.03 * dat_i$temp)]
	} else if(dat_i$bls == 0.7){
		# Apply EQ5
		art_dat[i, 'mo2' := 1.85 * exp(0.03 * dat_i$temp)]
	} else if(dat_i$bls == 1.0){
		# Apply EQ6
		art_dat[i, 'mo2' := 2.90 + (0.02 * dat_i$temp) + (0.0021 * dat_i$temp^2)]
	}
}

# # # The linear predictor function in equation 2 in the manuscript is mu = exp(...parameters...),
# # # which is equivalent to log(mu) = ...parameters...
art_dat[, log_mo2 := log(mo2)]



# # # Create a temporary model specification file and fit the model
# # # This is the final model from an iterative process in which, we tested several alternative models, 
# # # all of which were inferior to this as judged by DIC.
sink("mo2_params.txt")
cat("
model{
	#1A. Priors
	a1	~ dnorm(0, 25)
	b1	~ dnorm(0, 25)
	b1a	~ dnorm(0, 25)
	b2	~ dnorm(0, 25)
	b2a	~ dnorm(0, 25)
	b3	~ dnorm(0, 25)
	b4	~ dnorm(0, 25)
	
	# variance
	num   ~ dnorm(0, 0.0016)
	denom ~ dnorm(0, 1)
	sigma <- abs(num / denom)
	tau <- 1 / (sigma * sigma)
	#2. Likelihood
	for (i in 1:N){
		log_mo2[i] 			~ dnorm(mu_mo2[i], tau)	
		mu_mo2[i] 			= intercept[i] + beta_temp[i] + beta_bls[i] + beta_size[i] + beta_tempXbls[i]
		intercept[i] 		= a1
		beta_temp[i] 		= b1 * temp[i] + b1a *temp[i] * temp[i]
		beta_bls[i]  		= b2 * bls[i] + b2a * bls[i] * bls[i]
		beta_size[i] 		= b4 * lng[i]
		beta_tempXbls[i] 	= b3 * bls[i] * temp[i]
	}
}
",fill = TRUE)
sink()
		
params <- c("a1","b1", "b1a",  "b2", "b2a", "b3", "b4", "tau", "sigma")
inits <- function() {list(num=rnorm(1,0,25), denom=rnorm(1,0,1))}
jags_dat <- list(log_mo2=art_dat$log_mo2, lng=art_dat$lng, temp=art_dat$temp, bls=art_dat$bls, N=nrow(art_dat))
J1 <- jags(data = jags_dat, parameters = params, inits = inits, model = "mo2_params.txt", n.thin = 10, n.chains = 3, n.burnin = 10000, n.iter = 100000)
out <- J1$BUGSoutput


# # # Extract model parameters from posterior
out$mean

# # # Compare to equation 2 from the manuscript
# # # Note that values might differ slightly as this is MCMC based
# # # µ = exp(-0.39 + 0.05*T + 0.0002*T2 + 1.38*BLs + 0.28*BLs2 - 0.038*LT – 0.002*T*BLs)




